@@ -24,10 +24,13 @@
 
 //---------------------------------------------------------------------------
 #include "ZenLib/Utils.h"
+#if defined(MEDIAINFO_FILE_YES)
 #include "ZenLib/File.h"
+#endif //defined(MEDIAINFO_FILE_YES)
 #include "ZenLib/FileName.h"
 #include "MediaInfo/File__Analyze.h"
 #include "MediaInfo/MediaInfo_Config_MediaInfo.h"
+#include "MediaInfo/MediaInfo_Internal.h"
 #if MEDIAINFO_IBI
     #include "MediaInfo/Multiple/File_Ibi.h"
 #endif //MEDIAINFO_IBI
@@ -82,6 +85,66 @@ void File__Analyze::Streams_Finish_Global()
     if (IsSub)
         return;
 
+    TestDirectory();
+
+    #if MEDIAINFO_ADVANCED
+        if (MediaInfoLib::Config.ExternalMetaDataConfig_Get().empty()) // ExternalMetadata is used directly only if there is no ExternalMetadata config (=another format)
+        {
+            Ztring ExternalMetadata=MediaInfoLib::Config.ExternalMetadata_Get();
+            if (!ExternalMetadata.empty())
+            {
+                ZtringListList List;
+                List.Separator_Set(0, MediaInfoLib::Config.LineSeparator_Get());
+                List.Separator_Set(1, __T(";"));
+                List.Write(ExternalMetadata);
+
+                for (size_t i=0; i<List.size(); i++)
+                {
+                    // col 1&2 can be removed, conidered as "General;0"
+                    // 1: stream kind (General, Video, Audio, Text...)
+                    // 2: 0-based stream number
+                    // 3: field name
+                    // 4: field value
+                    // 5 (optional): replace instead of ignoring if field is already present (metadata from the file)
+                    if (List[i].size()<2 || List[i].size()>5)
+                    {
+                        MediaInfoLib::Config.Log_Send(0xC0, 0xFF, 0, "Invalid column size for external metadata");
+                        continue;
+                    }
+
+                    Ztring StreamKindZ=Ztring(List[i][0]).MakeLowerCase();
+                    stream_t StreamKind;
+                    size_t   Offset;
+                    if (List[i].size()<4)
+                    {
+                        StreamKind=Stream_General;
+                        Offset=2;
+                    }
+                    else
+                    {
+                        Offset=0;
+                             if (StreamKindZ==__T("general"))   StreamKind=Stream_General;
+                        else if (StreamKindZ==__T("video"))     StreamKind=Stream_Video;
+                        else if (StreamKindZ==__T("audio"))     StreamKind=Stream_Audio;
+                        else if (StreamKindZ==__T("text"))      StreamKind=Stream_Text;
+                        else if (StreamKindZ==__T("other"))     StreamKind=Stream_Other;
+                        else if (StreamKindZ==__T("image"))     StreamKind=Stream_Image;
+                        else if (StreamKindZ==__T("menu"))      StreamKind=Stream_Menu;
+                        else
+                        {
+                            MediaInfoLib::Config.Log_Send(0xC0, 0xFF, 0, "Invalid column 0 for external metadata");
+                            continue;
+                        }
+                    }
+                    size_t StreamPos=(size_t)List[i][1].To_int64u();
+                    bool ShouldReplace=List[i].size()>4-Offset && List[i][4-Offset].To_int64u();
+                    if (ShouldReplace || Retrieve_Const(StreamKind, StreamPos, List[i][2-Offset].To_UTF8().c_str()).empty())
+                        Fill(StreamKind, StreamPos, List[i][2-Offset].To_UTF8().c_str(), List[i][3-Offset], ShouldReplace);
+                }
+            }
+        }
+    #endif //MEDIAINFO_ADVANCED
+
     #if MEDIAINFO_ADVANCED
         //Default frame rate
         if (Count_Get(Stream_Video)==1 && Retrieve(Stream_Video, 0, Video_FrameRate).empty() && Config->File_DefaultFrameRate_Get())
@@ -93,7 +156,7 @@ void File__Analyze::Streams_Finish_Global()
     {
         if (Frame_Count_NotParsedIncluded!=(int64u)-1 && File_Offset+Buffer_Size==File_Size)
             Fill(Stream_Video, 0, Video_FrameCount, Frame_Count_NotParsedIncluded);
-        else if (Config->File_Names.size()>1 && IsRawStream)
+        else if (Config->File_Names.size()>1 && StreamSource==IsStream)
             Fill(Stream_Video, 0, Video_FrameCount, Config->File_Names.size());
         #if MEDIAINFO_IBIUSAGE
         else
@@ -121,6 +184,22 @@ void File__Analyze::Streams_Finish_Global()
         #endif //MEDIAINFO_IBIUSAGE
     }
 
+    //Exception
+    if (Retrieve(Stream_General, 0, General_Format)==__T("AC-3") && (Retrieve(Stream_General, 0, General_Format_Profile).find(__T("E-AC-3"))==0 || Retrieve(Stream_General, 0, General_Format_AdditionalFeatures).find(__T("Dep"))!=string::npos))
+    {
+        //Using AC-3 extensions + E-AC-3 extensions + "eb3" specific extension
+        Ztring Extensions=Retrieve(Stream_General, 0, General_Format_Extensions);
+        if (Extensions.find(__T(" eb3"))==string::npos)
+        {
+            Extensions+=__T(' ');
+            Extensions+=MediaInfoLib::Config.Format_Get(__T("E-AC-3"), InfoFormat_Extensions);
+            Extensions+=__T(" eb3");
+            Fill(Stream_General, 0, General_Format_Extensions, Extensions, true);
+            if (MediaInfoLib::Config.Legacy_Get())
+                Fill(Stream_General, 0, General_Codec_Extensions, Extensions, true);
+        }
+    }
+
     Streams_Finish_StreamOnly();
     Streams_Finish_StreamOnly();
     Streams_Finish_InterStreams();
@@ -130,11 +209,14 @@ void File__Analyze::Streams_Finish_Global()
     Streams_Finish_InterStreams();
     Streams_Finish_StreamOnly();
 
+    Config->File_ExpandSubs_Update((void**)(&Stream_More));
+
     if (!IsSub && !Config->File_IsReferenced_Get() && MediaInfoLib::Config.ReadByHuman_Get())
         Streams_Finish_HumanReadable();
 }
 
 //---------------------------------------------------------------------------
+#if defined(MEDIAINFO_FILE_YES)
 void File__Analyze::TestContinuousFileNames(size_t CountOfFiles, Ztring FileExtension, bool SkipComputeDelay)
 {
     if (IsSub || !Config->File_TestContinuousFileNames_Get())
@@ -263,7 +345,7 @@ void File__Analyze::TestContinuousFileNames(size_t CountOfFiles, Ztring FileExte
         return;
 
     Config->File_IsImageSequence=true;
-    if (IsRawStream)
+    if (StreamSource==IsStream)
         Frame_Count_NotParsedIncluded=Pos_Base;
     #if MEDIAINFO_DEMUX
         float64 Demux_Rate=Config->Demux_Rate_Get();
@@ -307,6 +389,10 @@ void File__Analyze::TestContinuousFileNames(size_t CountOfFiles, Ztring FileExte
         Fill (Stream_General, 0, General_FolderName_Last, FileName::Path_Get(Config->File_Names[Config->File_Names.size()-1]), true);
         Fill (Stream_General, 0, General_FileName_Last, FileName::Name_Get(Config->File_Names[Config->File_Names.size()-1]), true);
         Fill (Stream_General, 0, General_FileExtension_Last, FileName::Extension_Get(Config->File_Names[Config->File_Names.size()-1]), true);
+        if (Retrieve(Stream_General, 0, General_FileExtension_Last).empty())
+            Fill(Stream_General, 0, General_FileNameExtension_Last, Retrieve(Stream_General, 0, General_FileName_Last));
+        else
+            Fill(Stream_General, 0, General_FileNameExtension_Last, Retrieve(Stream_General, 0, General_FileName_Last)+__T('.')+Retrieve(Stream_General, 0, General_FileExtension_Last));
     }
 
     #if MEDIAINFO_ADVANCED
@@ -324,6 +410,142 @@ void File__Analyze::TestContinuousFileNames(size_t CountOfFiles, Ztring FileExte
         }
     #endif //MEDIAINFO_ADVANCED
 }
+#endif //defined(MEDIAINFO_FILE_YES)
+
+//---------------------------------------------------------------------------
+#if defined(MEDIAINFO_FILE_YES)
+// title-of-work/
+//     title-of-work.wav
+//     dpx/
+//         title-of-work_0086880.dpx
+//         title-of-work_0086881.dpx
+//         ... etc ...
+static void PotentialAudioNames_Scenario1(const Ztring& DpxName, Ztring& ContainerDirName, ZtringList& List)
+{
+    if (DpxName.size()<4)
+        return;
+
+    if (DpxName.substr(DpxName.size()-4)!=__T(".dpx"))
+        return;
+
+    size_t PathSeparator_Pos1=DpxName.find_last_of(__T("\\/"));
+    if (PathSeparator_Pos1==string::npos)
+        return;
+
+    size_t PathSeparator_Pos2=DpxName.find_last_of(__T("\\/"), PathSeparator_Pos1-1);
+    if (PathSeparator_Pos2==string::npos)
+        return;
+
+    size_t PathSeparator_Pos3=DpxName.find_last_of(__T("\\/"), PathSeparator_Pos2-1); //string::npos is accepted (relative path) 
+
+    size_t TitleSeparator_Pos=DpxName.find_last_of(__T('_'));
+    if (TitleSeparator_Pos==string::npos || TitleSeparator_Pos<=PathSeparator_Pos1)
+        return;
+
+    Ztring DirDpx=DpxName.substr(PathSeparator_Pos2+1, PathSeparator_Pos1-(PathSeparator_Pos2+1));
+    if (DirDpx!=__T("dpx"))
+        return;
+
+    Ztring TitleDpx=DpxName.substr(PathSeparator_Pos1+1, TitleSeparator_Pos-(PathSeparator_Pos1+1));
+    Ztring TitleDir=DpxName.substr(PathSeparator_Pos3+1, PathSeparator_Pos2-(PathSeparator_Pos3+1));
+    if (TitleDpx!=TitleDir)
+        return;
+
+    ContainerDirName=DpxName.substr(0, PathSeparator_Pos2+1);
+    List.push_back(ContainerDirName+TitleDpx+__T(".wav"));
+}
+void File__Analyze::TestDirectory()
+{
+    if (IsSub || !Config->File_TestDirectory_Get())
+        return;
+
+    if (Config->File_Names.size()<=1)
+        return;
+
+    Ztring ContainerDirName;
+    ZtringList List;
+    PotentialAudioNames_Scenario1(Config->File_Names[0], ContainerDirName, List);
+    bool IsModified=false;
+    for (size_t i=0; i<List.size(); i++)
+    {
+        MediaInfo_Internal MI;
+        if (MI.Open(List[i]))
+        {
+            IsModified=true;
+            Ztring AudioFileName=MI.Get(Stream_General, 0, General_CompleteName);
+            for (size_t StreamKind=Stream_General+1; StreamKind<Stream_Max; StreamKind++)
+                for (size_t StreamPos=0; StreamPos<MI.Count_Get((stream_t)StreamKind); StreamPos++)
+                {
+                    Stream_Prepare((stream_t)StreamKind);
+                    Merge(MI, (stream_t)StreamKind, StreamPos_Last, StreamPos);
+                    if (AudioFileName.size()>ContainerDirName.size())
+                        Fill((stream_t)StreamKind, StreamPos_Last, "Source", AudioFileName.substr(ContainerDirName.size()));
+                    Fill((stream_t)StreamKind, StreamPos_Last, "MuxingMode", MI.Get(Stream_General, 0, General_Format));
+                    if (Retrieve_Const((stream_t)StreamKind, StreamPos_Last, "Encoded_Application").empty())
+                        Fill((stream_t)StreamKind, StreamPos_Last, "Encoded_Application", MI.Get(Stream_General, 0, General_Encoded_Application));
+                    if (Retrieve_Const((stream_t)StreamKind, StreamPos_Last, "Encoded_Library").empty())
+                        Fill((stream_t)StreamKind, StreamPos_Last, "Encoded_Library", MI.Get(Stream_General, 0, General_Encoded_Library));
+                }
+            #if MEDIAINFO_ADVANCED
+                if (!Config->File_IgnoreSequenceFileSize_Get())
+            #endif //MEDIAINFO_ADVANCED
+            {
+                File_Size+=MI.Get(Stream_General, 0, General_FileSize).To_int64u();
+            }
+        }
+    }
+    if (IsModified)
+    {
+        Ztring VideoFileName=Retrieve(Stream_General, 0, General_CompleteName);
+        Ztring VideoFileName_Last=Retrieve(Stream_General, 0, General_CompleteName_Last);
+        Ztring VideoMuxingMode=Retrieve_Const(Stream_General, 0, General_Format);
+        if (VideoFileName.size()>ContainerDirName.size())
+            Fill(Stream_Video, 0, "Source", VideoFileName.substr(ContainerDirName.size()));
+        if (VideoFileName_Last.size()>ContainerDirName.size())
+            Fill(Stream_Video, 0, "Source_Last", VideoFileName_Last.substr(ContainerDirName.size()));
+        Fill(Stream_Video, 0, Video_MuxingMode, VideoMuxingMode);
+
+        Fill(Stream_General, 0, General_CompleteName, ContainerDirName, true);
+        Fill(Stream_General, 0, General_FileSize, File_Size, 10, true);
+        Fill(Stream_General, 0, General_Format, "Directory", Unlimited, true, true);
+
+        Clear(Stream_General, 0, General_CompleteName_Last);
+        Clear(Stream_General, 0, General_FolderName_Last);
+        Clear(Stream_General, 0, General_FileName_Last);
+        Clear(Stream_General, 0, General_FileNameExtension_Last);
+        Clear(Stream_General, 0, General_FileExtension_Last);
+        Clear(Stream_General, 0, General_Format_String);
+        Clear(Stream_General, 0, General_Format_Info);
+        Clear(Stream_General, 0, General_Format_Url);
+        Clear(Stream_General, 0, General_Format_Commercial);
+        Clear(Stream_General, 0, General_Format_Commercial_IfAny);
+        Clear(Stream_General, 0, General_Format_Version);
+        Clear(Stream_General, 0, General_Format_Profile);
+        Clear(Stream_General, 0, General_Format_Level);
+        Clear(Stream_General, 0, General_Format_Compression);
+        Clear(Stream_General, 0, General_Format_Settings);
+        Clear(Stream_General, 0, General_Format_AdditionalFeatures);
+        Clear(Stream_General, 0, General_InternetMediaType);
+        Clear(Stream_General, 0, General_Duration);
+        Clear(Stream_General, 0, General_Encoded_Application);
+        Clear(Stream_General, 0, General_Encoded_Application_String);
+        Clear(Stream_General, 0, General_Encoded_Application_CompanyName);
+        Clear(Stream_General, 0, General_Encoded_Application_Name);
+        Clear(Stream_General, 0, General_Encoded_Application_Version);
+        Clear(Stream_General, 0, General_Encoded_Application_Url);
+        Clear(Stream_General, 0, General_Encoded_Library);
+        Clear(Stream_General, 0, General_Encoded_Library_String);
+        Clear(Stream_General, 0, General_Encoded_Library_CompanyName);
+        Clear(Stream_General, 0, General_Encoded_Library_Name);
+        Clear(Stream_General, 0, General_Encoded_Library_Version);
+        Clear(Stream_General, 0, General_Encoded_Library_Date);
+        Clear(Stream_General, 0, General_Encoded_Library_Settings);
+        Clear(Stream_General, 0, General_Encoded_OperatingSystem);
+        Clear(Stream_General, 0, General_FrameCount);
+        Clear(Stream_General, 0, General_FrameRate);
+    }
+}
+#endif //defined(MEDIAINFO_FILE_YES)
 
 //---------------------------------------------------------------------------
 #if MEDIAINFO_FIXITY
@@ -384,6 +606,10 @@ void File__Analyze::Streams_Finish_StreamOnly()
 //---------------------------------------------------------------------------
 void File__Analyze::Streams_Finish_StreamOnly(stream_t StreamKind, size_t Pos)
 {
+    //Format
+    if (Retrieve_Const(StreamKind, Pos, Fill_Parameter(StreamKind, Generic_Format)).empty())
+        Fill(StreamKind, Pos, Fill_Parameter(StreamKind, Generic_Format), Retrieve_Const(StreamKind, Pos, Fill_Parameter(StreamKind, Generic_CodecID)));
+
     //BitRate from Duration and StreamSize
     if (StreamKind!=Stream_General && StreamKind!=Stream_Other && StreamKind!=Stream_Menu && Retrieve(StreamKind, Pos, "BitRate").empty() && !Retrieve(StreamKind, Pos, Fill_Parameter(StreamKind, Generic_StreamSize)).empty() && !Retrieve(StreamKind, Pos, Fill_Parameter(StreamKind, Generic_Duration)).empty())
     {
@@ -426,7 +652,7 @@ void File__Analyze::Streams_Finish_StreamOnly(stream_t StreamKind, size_t Pos)
         int64u BitRate=Retrieve(StreamKind, Pos, "BitRate").To_int64u();
         int64u StreamSize=Retrieve(StreamKind, Pos, Fill_Parameter(StreamKind, Generic_StreamSize)).To_int64u();
         if (BitRate>0 && StreamSize>0)
-            Fill(StreamKind, Pos, Fill_Parameter(StreamKind, Generic_Duration), StreamSize*8*1000/BitRate);
+            Fill(StreamKind, Pos, Fill_Parameter(StreamKind, Generic_Duration), ((float64)StreamSize)*8*1000/BitRate, 0);
     }
 
     //StreamSize from BitRate and Duration
@@ -463,7 +689,7 @@ void File__Analyze::Streams_Finish_StreamOnly_General(size_t StreamPos)
             {
                 ZtringList ValidExtensions;
                 ValidExtensions.Separator_Set(0, __T(" "));
-                ValidExtensions.Write(FormatList.Get(Format->first, InfoFormat_Extensions));
+                ValidExtensions.Write(Retrieve(Stream_General, StreamPos, General_Format_Extensions));
                 if (!ValidExtensions.empty() && ValidExtensions.Find(Extension)==string::npos)
                     Fill(Stream_General, StreamPos, "FileExtension_Invalid", ValidExtensions.Read());
             }
@@ -485,20 +711,41 @@ void File__Analyze::Streams_Finish_StreamOnly_Video(size_t Pos)
     if (Retrieve(Stream_Video, Pos, Video_FrameCount).empty())
     {
         int64s Duration=Retrieve(Stream_Video, Pos, Video_Duration).To_int64s();
+        bool DurationFromGeneral;
         if (Duration==0)
+        {
             Duration=Retrieve(Stream_General, 0, General_Duration).To_int64s();
+            DurationFromGeneral=Retrieve(Stream_General, 0, General_Format)!=Retrieve(Stream_Video, Pos, Audio_Format);
+        }
+        else
+            DurationFromGeneral=false;
         float64 FrameRate=Retrieve(Stream_Video, Pos, Video_FrameRate).To_float64();
         if (Duration && FrameRate)
-           Fill(Stream_Video, Pos, Video_FrameCount, Duration*FrameRate/1000, 0);
+        {
+            Fill(Stream_Video, Pos, Video_FrameCount, Duration*FrameRate/1000, 0);
+            if (DurationFromGeneral && Retrieve_Const(Stream_Audio, Pos, Audio_Format)!=Retrieve_Const(Stream_General, 0, General_Format))
+            {
+                Fill(Stream_Video, Pos, "FrameCount_Source", "General_Duration");
+                Fill_SetOptions(Stream_Video, Pos, "FrameCount_Source", "N NTN");
+            }
+        }
     }
 
     //Duration from FrameCount and FrameRate
     if (Retrieve(Stream_Video, Pos, Video_Duration).empty())
     {
         int64u FrameCount=Retrieve(Stream_Video, Pos, Video_FrameCount).To_int64u();
-        float64 FrameRate=Retrieve(Stream_Video, Pos, "FrameRate").To_float64();
+        float64 FrameRate=Retrieve(Stream_Video, Pos, Video_FrameRate).To_float64();
         if (FrameCount && FrameRate)
-           Fill(Stream_Video, Pos, Video_Duration, FrameCount/FrameRate*1000, 0);
+        {
+            Fill(Stream_Video, Pos, Video_Duration, FrameCount/FrameRate*1000, 0);
+            Ztring Source=Retrieve(Stream_Video, Pos, "FrameCount_Source");
+            if (!Source.empty())
+            {
+                Fill(Stream_Video, Pos, "Duration_Source", Source);
+                Fill_SetOptions(Stream_Video, Pos, "Duration_Source", "N NTN");
+            }
+        }
     }
 
     //FrameRate from FrameCount and Duration
@@ -553,6 +800,64 @@ void File__Analyze::Streams_Finish_StreamOnly_Video(size_t Pos)
     }
 
     //Commercial name
+    if ((Retrieve(Stream_Video, Pos, Video_BitDepth).empty() || Retrieve(Stream_Video, Pos, Video_BitDepth)==__T("10")) //e.g. ProRes has not bitdepth info
+     && Retrieve(Stream_Video, Pos, Video_ChromaSubsampling)==__T("4:2:0")
+     && (Retrieve(Stream_Video, Pos, Video_colour_description_present).empty() || //From  CFF: "colour_description_present_flag SHALL be set to 1 if the color parameters from [R709] are not used."
+       ( Retrieve(Stream_Video, Pos, Video_colour_primaries)==__T("BT.2020")
+      && Retrieve(Stream_Video, Pos, Video_transfer_characteristics)==__T("PQ")
+      && Retrieve(Stream_Video, Pos, Video_matrix_coefficients).find(__T("BT.2020"))==0))
+     && !Retrieve(Stream_Video, Pos, "MasteringDisplay_ColorPrimaries").empty()
+     // && !Retrieve(Stream_Video, Pos, "MaxCLL").empty()
+     // && !Retrieve(Stream_Video, Pos, "MaxFALL").empty() // MaxCLL & MaxFALL are required except if not available so not required in practice https://www.cta.tech/News/Press-Releases/2015/August/CEA-Defines-%E2%80%98HDR-Compatible%E2%80%99-Displays.aspx https://www.ultrahdforum.org/wp-content/uploads/2016/04/Ultra-HD-Forum-Deployment-Guidelines-V1.1-Summer-2016.pdf
+        )
+    {
+        //We actually fill HDR10/HDR10+ by default, so it will be removed below if not fitting in the color related rules
+    }
+    else if (!Retrieve_Const(Stream_Video, Pos, Video_HDR_Format_Compatibility).empty())
+    {
+    }
+    if (Retrieve(Stream_Video, Pos, Video_HDR_Format_String).empty())
+    {
+        ZtringList Summary;
+        Summary.Separator_Set(0, __T(" / "));
+        Summary.Write(Retrieve(Stream_Video, Pos, Video_HDR_Format));
+        ZtringList Commercial=Summary;
+        if (!Summary.empty())
+        {
+            ZtringList HDR_Format_Compatibility;
+            HDR_Format_Compatibility.Separator_Set(0, __T(" / "));
+            HDR_Format_Compatibility.Write(Retrieve(Stream_Video, Pos, Video_HDR_Format_Compatibility));
+            HDR_Format_Compatibility.resize(Summary.size());
+            ZtringList ToAdd;
+            ToAdd.Separator_Set(0, __T(" / "));
+            for (size_t i=Video_HDR_Format_String+1; i<=Video_HDR_Format_Settings; i++)
+            {
+                ToAdd.Write(Retrieve(Stream_Video, Pos, i));
+                ToAdd.resize(Summary.size());
+                for (size_t j=0; j<Summary.size(); j++)
+                {
+                    if (!ToAdd[j].empty())
+                    {
+                        switch (i)
+                        {
+                            case Video_HDR_Format_Version: Summary[j]+=__T(", Version "); break;
+                            case Video_HDR_Format_Level: Summary[j]+=__T('.'); break;
+                            default: Summary[j] += __T(", ");
+                        }
+                        Summary[j]+=ToAdd[j];
+                    }
+                }
+            }
+            for (size_t j=0; j<Summary.size(); j++)
+                if (!HDR_Format_Compatibility[j].empty())
+                {
+                    Summary[j]+=__T(", ")+HDR_Format_Compatibility[j]+__T(" compatible");
+                    Commercial[j]=HDR_Format_Compatibility[j].substr(0, HDR_Format_Compatibility[j].find(__T(' ')));
+                }
+            Fill(Stream_Video, Pos, Video_HDR_Format_String, Summary.Read());
+            Fill(Stream_Video, Pos, Video_HDR_Format_Commercial, Commercial.Read());
+        }
+    }
     #if defined(MEDIAINFO_VC3_YES)
         if (Retrieve(Stream_Video, Pos, Video_Format_Commercial_IfAny).empty() && Retrieve(Stream_Video, Pos, Video_Format)==__T("VC-3") && Retrieve(Stream_Video, Pos, Video_Format_Profile).find(__T("HD"))==0)
         {
@@ -702,11 +1007,24 @@ void File__Analyze::Streams_Finish_StreamOnly_Audio(size_t Pos)
     if (Retrieve(Stream_Audio, Pos, Audio_SamplingCount).empty())
     {
         int64s Duration=Retrieve(Stream_Audio, Pos, Audio_Duration).To_int64s();
+        bool DurationFromGeneral; 
         if (Duration==0)
+        {
             Duration=Retrieve(Stream_General, 0, General_Duration).To_int64s();
+            DurationFromGeneral=Retrieve(Stream_General, 0, General_Format)!=Retrieve(Stream_Audio, Pos, Audio_Format);
+        }
+        else
+            DurationFromGeneral=false;
         float SamplingRate=Retrieve(Stream_Audio, Pos, Audio_SamplingRate).To_float32();
         if (Duration && SamplingRate)
+        {
             Fill(Stream_Audio, Pos, Audio_SamplingCount, ((float64)Duration)/1000*SamplingRate, 0);
+            if (DurationFromGeneral && Retrieve_Const(Stream_Audio, Pos, Audio_Format)!=Retrieve_Const(Stream_General, 0, General_Format))
+            {
+                Fill(Stream_Audio, Pos, "SamplingCount_Source", "General_Duration");
+                Fill_SetOptions(Stream_Audio, Pos, "SamplingCount_Source", "N NTN");
+            }
+        }
     }
 
     //Frame count
@@ -763,7 +1081,15 @@ void File__Analyze::Streams_Finish_StreamOnly_Audio(size_t Pos)
     {
         int64u Duration=Retrieve(Stream_Audio, Pos, Audio_SamplingCount).To_int64u()*1000/Retrieve(Stream_Audio, Pos, Audio_SamplingRate).To_int64u();
         if (Duration)
-           Fill(Stream_Audio, Pos, Audio_Duration, Duration);
+        {
+            Fill(Stream_Audio, Pos, Audio_Duration, Duration);
+            Ztring Source=Retrieve(Stream_Audio, Pos, "SamplingCount_Source");
+            if (!Source.empty())
+            {
+                Fill(Stream_Audio, Pos, "Duration_Source", Source);
+                Fill_SetOptions(Stream_Audio, Pos, "Duration_Source", "N NTN");
+            }
+        }
     }
 
     //Stream size
@@ -771,8 +1097,11 @@ void File__Analyze::Streams_Finish_StreamOnly_Audio(size_t Pos)
     {
         int64u Duration=Retrieve(Stream_Audio, Pos, Audio_Duration).To_int64u();
         int64u BitRate=Retrieve(Stream_Audio, Pos, Audio_BitRate).To_int64u();
+        int64u BitRate_Encoded=Retrieve(Stream_Audio, Pos, Audio_BitRate_Encoded).To_int64u();
         if (Duration && BitRate)
             Fill(Stream_Audio, Pos, Audio_StreamSize, Duration*BitRate/8/1000);
+        if (Duration && BitRate_Encoded)
+            Fill(Stream_Audio, Pos, Audio_StreamSize_Encoded, Duration*BitRate_Encoded/8/1000);
     }
 
     //CBR/VBR
@@ -862,8 +1191,11 @@ void File__Analyze::Streams_Finish_InterStreams()
     }
 
     //OverallBitRate if we have one Audio stream with bitrate
-    if (Retrieve(Stream_General, 0, General_Duration).empty() && Retrieve(Stream_General, 0, General_OverallBitRate).empty() && Count_Get(Stream_Video)==0 && Count_Get(Stream_Audio)==1 && Retrieve(Stream_Audio, 0, Audio_BitRate).To_int64u()!=0 && (Retrieve(Stream_General, 0, General_Format)==Retrieve(Stream_Audio, 0, Audio_Format) || !Retrieve(Stream_General, 0, General_HeaderSize).empty()))
-        Fill(Stream_General, 0, General_OverallBitRate, Retrieve(Stream_Audio, 0, Audio_BitRate));
+    if (Retrieve(Stream_General, 0, General_Duration).empty() && Retrieve(Stream_General, 0, General_OverallBitRate).empty() && Count_Get(Stream_Video) == 0 && Count_Get(Stream_Audio) == 1 && Retrieve(Stream_Audio, 0, Audio_BitRate).To_int64u() != 0 && (Retrieve(Stream_General, 0, General_Format) == Retrieve(Stream_Audio, 0, Audio_Format) || !Retrieve(Stream_General, 0, General_HeaderSize).empty()))
+    {
+        const Ztring& EncodedBitRate=Retrieve_Const(Stream_Audio, 0, Audio_BitRate_Encoded);
+        Fill(Stream_General, 0, General_OverallBitRate, EncodedBitRate.empty()?Retrieve_Const(Stream_Audio, 0, Audio_BitRate):EncodedBitRate);
+    }
 
     //OverallBitRate if Duration
     if (Retrieve(Stream_General, 0, General_OverallBitRate).empty() && Retrieve(Stream_General, 0, General_Duration).To_int64u()!=0 && !Retrieve(Stream_General, 0, General_FileSize).empty())
@@ -904,7 +1236,7 @@ void File__Analyze::Streams_Finish_InterStreams()
         double TextBitRate_Ratio   =0.98;  //Default container overhead=2%
         int32u TextBitRate_Minus   =2000;  //2000 bps because of a "classic" stream overhead
         //Specific value depends of Container
-        if (IsRawStream)
+        if (StreamSource==IsStream)
         {
             GeneralBitRate_Ratio=1;
             GeneralBitRate_Minus=0;
@@ -1017,7 +1349,7 @@ void File__Analyze::Streams_Finish_InterStreams()
                 if (Duration)
                 {
                     int64u StreamSize=float64_int64s(VideoBitRate/8*Duration/1000);
-                    if (IsRawStream && File_Size!=(int64u)-1 && StreamSize>=File_Size*0.99)
+                    if (StreamSource==IsStream && File_Size!=(int64u)-1 && StreamSize>=File_Size*0.99)
                         StreamSize=File_Size;
                     Fill(Stream_Video, 0, Video_StreamSize, StreamSize);
                 }
@@ -1124,8 +1456,8 @@ void File__Analyze::Streams_Finish_HumanReadable()
 //---------------------------------------------------------------------------
 void File__Analyze::Streams_Finish_HumanReadable_PerStream(stream_t StreamKind, size_t StreamPos, size_t Parameter)
 {
-    Ztring ParameterName=Retrieve(StreamKind, StreamPos, Parameter, Info_Name);
-    Ztring Value=Retrieve(StreamKind, StreamPos, Parameter, Info_Text);
+    const Ztring ParameterName=Retrieve(StreamKind, StreamPos, Parameter, Info_Name);
+    const Ztring Value=Retrieve(StreamKind, StreamPos, Parameter, Info_Text);
 
     //Strings
     const Ztring &List_Measure_Value=MediaInfoLib::Config.Info_Get(StreamKind).Read(Parameter, Info_Measure);
@@ -1162,6 +1494,12 @@ void File__Analyze::Streams_Finish_HumanReadable_PerStream(stream_t StreamKind,
          && !Retrieve(StreamKind, StreamPos, Video_FrameRate_Original_Num).empty()
          && !Retrieve(StreamKind, StreamPos, Video_FrameRate_Original_Den).empty())
             Fill(Stream_Video, StreamPos, Video_FrameRate_Original_String, MediaInfoLib::Config.Language_Get(Retrieve(StreamKind, StreamPos, Video_FrameRate_Original)+__T(" (")+Retrieve(StreamKind, StreamPos, Video_FrameRate_Original_Num)+__T("/")+Retrieve(StreamKind, StreamPos, Video_FrameRate_Original_Den)+__T(")"), __T(" fps")), true);
+        if (StreamKind==Stream_Other
+         && Parameter==Other_FrameRate
+         && !Retrieve(StreamKind, StreamPos, Other_FrameRate).empty()
+         && !Retrieve(StreamKind, StreamPos, Other_FrameRate_Num).empty()
+         && !Retrieve(StreamKind, StreamPos, Other_FrameRate_Den).empty())
+            Fill(Stream_Other, StreamPos, Other_FrameRate_String, MediaInfoLib::Config.Language_Get(Retrieve(StreamKind, StreamPos, Other_FrameRate)+__T(" (")+Retrieve(StreamKind, StreamPos, Other_FrameRate_Num)+__T("/")+Retrieve(StreamKind, StreamPos, Other_FrameRate_Den)+__T(")"), __T(" fps")), true);
     }
 
     //BitRate_Mode / OverallBitRate_Mode
@@ -1177,7 +1515,7 @@ void File__Analyze::Streams_Finish_HumanReadable_PerStream(stream_t StreamKind,
         for (size_t Pos=0; Pos<List.size(); Pos++)
             List[Pos]=MediaInfoLib::Config.Language_Get(Ztring(__T("BitRate_Mode_"))+List[Pos]);
 
-        Ztring Translated=List.Read();
+        const Ztring Translated=List.Read();
         Fill(StreamKind, StreamPos, StreamKind==Stream_General?"OverallBitRate_Mode/String":"BitRate_Mode/String", Translated.find(__T("BitRate_Mode_"))?Translated:Value);
     }
 
@@ -1238,20 +1576,17 @@ void File__Analyze::Streams_Finish_HumanReadable_PerStream(stream_t StreamKind,
     //Format_Settings_Matrix
     if (StreamKind==Stream_Video && Parameter==Video_Format_Settings_Matrix)
     {
-        Ztring Translated=MediaInfoLib::Config.Language_Get(Ztring(__T("Format_Settings_Matrix_"))+Value);
-        Fill(Stream_Video, StreamPos, Video_Format_Settings_Matrix_String, Translated.find(__T("Format_Settings_Matrix_"))?Translated:Value);
+        Fill(Stream_Video, StreamPos, Video_Format_Settings_Matrix_String, MediaInfoLib::Config.Language_Get_Translate(__T("Format_Settings_Matrix_"), Value));
     }
 
     //Scan type
     if (StreamKind==Stream_Video && Parameter==Video_ScanType)
     {
-        Ztring Translated=MediaInfoLib::Config.Language_Get(Ztring(__T("Interlaced_"))+Value);
-        Fill(Stream_Video, StreamPos, Video_ScanType_String, Translated.find(__T("Interlaced_"))?Translated:Value);
+        Fill(Stream_Video, StreamPos, Video_ScanType_String, MediaInfoLib::Config.Language_Get_Translate(__T("Interlaced_"), Value));
     }
     if (StreamKind==Stream_Video && Parameter==Video_ScanType_Original)
     {
-        Ztring Translated=MediaInfoLib::Config.Language_Get(Ztring(__T("Interlaced_"))+Value);
-        Fill(Stream_Video, StreamPos, Video_ScanType_Original_String, Translated.find(__T("Interlaced_"))?Translated:Value);
+        Fill(Stream_Video, StreamPos, Video_ScanType_Original_String, MediaInfoLib::Config.Language_Get_Translate(__T("Interlaced_"), Value));
     }
     if (StreamKind==Stream_Video && Parameter==Video_ScanType_StoreMethod)
     {
@@ -1265,18 +1600,15 @@ void File__Analyze::Streams_Finish_HumanReadable_PerStream(stream_t StreamKind,
     //Scan order
     if (StreamKind==Stream_Video && Parameter==Video_ScanOrder)
     {
-        Ztring Translated=MediaInfoLib::Config.Language_Get(Ztring(__T("Interlaced_"))+Value);
-        Fill(Stream_Video, StreamPos, Video_ScanOrder_String, Translated.find(__T("Interlaced_"))?Translated:Value);
+        Fill(Stream_Video, StreamPos, Video_ScanOrder_String, MediaInfoLib::Config.Language_Get_Translate(__T("Interlaced_"), Value));
     }
     if (StreamKind==Stream_Video && Parameter==Video_ScanOrder_Stored)
     {
-        Ztring Translated=MediaInfoLib::Config.Language_Get(Ztring(__T("Interlaced_"))+Value);
-        Fill(Stream_Video, StreamPos, Video_ScanOrder_Stored_String, Translated.find(__T("Interlaced_"))?Translated:Value);
+        Fill(Stream_Video, StreamPos, Video_ScanOrder_Stored_String, MediaInfoLib::Config.Language_Get_Translate(__T("Interlaced_"), Value));
     }
     if (StreamKind==Stream_Video && Parameter==Video_ScanOrder_Original)
     {
-        Ztring Translated=MediaInfoLib::Config.Language_Get(Ztring(__T("Interlaced_"))+Value);
-        Fill(Stream_Video, StreamPos, Video_ScanOrder_Original_String, Translated.find(__T("Interlaced_"))?Translated:Value);
+        Fill(Stream_Video, StreamPos, Video_ScanOrder_Original_String, MediaInfoLib::Config.Language_Get_Translate(__T("Interlaced_"), Value));
     }
 
     //Interlacement
@@ -1294,29 +1626,25 @@ void File__Analyze::Streams_Finish_HumanReadable_PerStream(stream_t StreamKind,
     //FrameRate_Mode
     if (StreamKind==Stream_Video && Parameter==Video_FrameRate_Mode)
     {
-        Ztring Translated=MediaInfoLib::Config.Language_Get(Ztring(__T("FrameRate_Mode_"))+Value);
-        Fill(Stream_Video, StreamPos, Video_FrameRate_Mode_String, Translated.find(__T("FrameRate_Mode_"))?Translated:Value);
+        Fill(Stream_Video, StreamPos, Video_FrameRate_Mode_String, MediaInfoLib::Config.Language_Get_Translate(__T("FrameRate_Mode_"), Value));
     }
 
     //Compression_Mode
     if (Parameter==Fill_Parameter(StreamKind, Generic_Compression_Mode))
     {
-        Ztring Translated=MediaInfoLib::Config.Language_Get(Ztring(__T("Compression_Mode_"))+Value);
-        Fill(StreamKind, StreamPos, Fill_Parameter(StreamKind, Generic_Compression_Mode_String), Translated.find(__T("Compression_Mode_"))?Translated:Value);
+        Fill(StreamKind, StreamPos, Fill_Parameter(StreamKind, Generic_Compression_Mode_String), MediaInfoLib::Config.Language_Get_Translate(__T("Compression_Mode_"), Value));
     }
 
     //Delay_Source
     if (Parameter==Fill_Parameter(StreamKind, Generic_Delay_Source))
     {
-        Ztring Translated=MediaInfoLib::Config.Language_Get(Ztring(__T("Delay_Source_"))+Value);
-        Fill(StreamKind, StreamPos, Fill_Parameter(StreamKind, Generic_Delay_Source_String), Translated.find(__T("Delay_Source_"))?Translated:Value);
+        Fill(StreamKind, StreamPos, Fill_Parameter(StreamKind, Generic_Delay_Source_String), MediaInfoLib::Config.Language_Get_Translate(__T("Delay_Source_"), Value));
     }
 
     //Gop_OpenClosed
     if (StreamKind==Stream_Video && (Parameter==Video_Gop_OpenClosed || Parameter==Video_Gop_OpenClosed_FirstFrame))
     {
-        Ztring Translated=MediaInfoLib::Config.Language_Get(Ztring(__T("Gop_OpenClosed_"))+Value);
-        Fill(Stream_Video, StreamPos, Parameter+1, Translated.find(__T("Gop_OpenClosed_"))?Translated:Value, true);
+        Fill(Stream_Video, StreamPos, Parameter+1, MediaInfoLib::Config.Language_Get_Translate(__T("Gop_OpenClosed_"), Value), true);
     }
 }
 
